import pygame
from random import randint

WIDTH = 600
HEIGHT = 600

BLACK =  (  0,   0,   0)
WHITE =  (255, 255, 255)
BLUE =   (  0,   0, 255)
GREEN =  (  0, 255,   0)
RED =    (255,   0,   0)
YELLOW = (255, 216,   0)

pygame.mixer.init(frequency=22050, size=-16, channels=2, buffer=512)
pygame.init()
clock = pygame.time.Clock()
screen = pygame.display.set_mode([WIDTH, HEIGHT])
screen.fill(BLACK)

# load graphics...
lander = pygame.image.load("lander.bmp")
lander_rect = lander.get_rect()
lander_rect.x = randint(1, WIDTH - lander.get_rect().width)
lander_rect.y = 0

surface = pygame.image.load("surface.bmp")
surface_rect = surface.get_rect();
surface_rect.y = HEIGHT - surface_rect.height

pad = pygame.image.load("landing_pad.png")
pad_rect = pad.get_rect();
pad_rect.x = randint(1, WIDTH - pad.get_rect().width)
pad_rect.y = HEIGHT - (pad.get_rect().height + 3)

flame1 = pygame.image.load("flame.png")
flame1_rect = flame1.get_rect();
flame2 = pygame.image.load("flame2.png")
flame2_rect = flame2.get_rect();

# load sound...
pygame.mixer.set_num_channels(10)
sound = pygame.mixer.Sound("thrust.wav")
voice = pygame.mixer.Channel(8)

sound_explode = pygame.mixer.Sound("explode.wav")
sound_land = pygame.mixer.Sound("land.wav")

pygame.mixer.music.load("music.mp3")
pygame.mixer.music.play()

# load font...
myfont = pygame.font.SysFont("monospace", 15)

# setup paramater and state...
GRAVITY = 0.1
THRUST = -0.2

DRAG = 0.01
STEER = 0.05

NUMBER_OF_STARS = 500

velocity_x = 0
velocity_y = 0.5
speed = [velocity_x, velocity_y]
fuel = 100.0
score = 0
end_score = 0
multiplier = 1

flame_toggle = True
thrust_engaged = False
explode_toggle = True

star_positions = []
for n in range(NUMBER_OF_STARS):
    star_positions.append ([randint(1,WIDTH), randint(1, HEIGHT), randint(1, 4)])

game_state = "PLAYING"
fps = 30

# the game...
while 1:
    pygame.event.pump()
    clock.tick(fps) #frames per second

    # capture keyboard input...
    keys = pygame.key.get_pressed()

    # check for exit (ESC)...
    if keys[pygame.K_ESCAPE] or keys[pygame.K_q]:
        pygame.quit()
        sys.exit(0)
        
    # update game (if running)...
    if game_state == "PLAYING":
        # apply gravity...
        velocity_y = velocity_y + GRAVITY

        if fuel > 0 and (keys[pygame.K_SPACE] or keys[pygame.K_UP]):
            velocity_y = velocity_y + THRUST
            thrust_engaged = True
            fuel = fuel - .3
        else:
            thrust_engaged = False

        if keys[pygame.K_LEFT]:
            velocity_x = velocity_x - STEER
        elif keys[pygame.K_RIGHT]:
            velocity_x = velocity_x + STEER
        
        if velocity_x > 0:
            velocity_x = velocity_x - DRAG

        if velocity_x < 0:
            velocity_x = velocity_x + DRAG
        
        if velocity_y > 5:
            velocity_y = 5

        if velocity_y < -3:
            velocity_y = -3

        if velocity_x > 1.5:
            velocity_x = 1.5

        if velocity_x < -1.5:
            velocity_x = -1.5

        # bounce off the sides...
        if lander_rect.x < 0 or lander_rect.x + lander_rect.width > WIDTH:
            velocity_x = velocity_x * -1
        
        # check lader has landed or crahsed...
        if (lander_rect.y + lander_rect.height >= pad_rect.y
           and lander_rect.x > pad_rect.x
           and lander_rect.x < pad_rect.x + pad_rect.width - lander_rect.width):
            if velocity_y > 1.5:
                game_state = "CRASHED"
                thrust_engaged = False
                fps = 30
                multiplier = 1
                end_score = score
                score = 0
                sound_explode.play()
                
            else:
                game_state = "LANDED"
                thrust_engaged = False
                score = score + 500 * int(fuel)
                fps = fps + 10
                multiplier = multiplier + 1
                end_score = score
                sound_land.play()
                
        elif lander_rect.y + lander_rect.height >= HEIGHT - 8:
            game_state = "CRASHED"
            thrust_engaged = False
            multiplier = 1
            fps = 30
            end_score = score
            score = 0
            sound_explode.play()

            
        else:
            score = score + round(velocity_y * 100) * multiplier
            speed = (round(velocity_x), round(velocity_y))
            lander_rect = lander_rect.move(speed)
    
    else:
        if keys[pygame.K_SPACE]:
            velocity_x = 0
            velocity_y = 0.5
            speed = [velocity_x, velocity_y]
            fuel = 100.0        
            lander = pygame.image.load("lander.bmp")
            lander_rect = lander.get_rect()
            lander_rect.x = randint(1, WIDTH - pad.get_rect().width)
            lander_rect.y = 0
            pad_rect.x = randint(1, WIDTH - pad.get_rect().width)
            game_state = "PLAYING"
    
    # Draw stuff...
    screen.fill(BLACK)

    if game_state == "CRASHED":
        label = myfont.render("CRASHED!", 1, YELLOW)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , HEIGHT / 2))
        label = myfont.render("YOUR SCORE: " + str(int(end_score)), 1, YELLOW)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 24))
        colour = [randint(1, 255),randint(1, 255),randint(1, 255)]
        label = myfont.render("HIR THE 'SPACE' BAR TO TRY AGAIN", 1, colour)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 48))
        label = myfont.render("OR PRESS 'Q' TO QUIT", 1, WHITE)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 72))

        explode_toggle = not explode_toggle
        if explode_toggle:
            lander = pygame.image.load("explode1.png")
        else:
            lander = pygame.image.load("explode2.png")
        
        
    elif game_state == "LANDED":
        label = myfont.render("WELL DONE! YOU'VE LANDED!", 1, YELLOW)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , HEIGHT / 2))
        label = myfont.render("YOUR SCORE: " + str(int(end_score)), 1, YELLOW)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 24))
        colour = [randint(1, 255),randint(1, 255),randint(1, 255)]
        label = myfont.render("HIT THE 'SPACE' BAR FOR NEXT ROUND!", 1, colour)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 48))
        label = myfont.render("OR PRESS 'Q' TO QUIT", 1, WHITE)
        screen.blit(label, ((WIDTH / 2) - (label.get_rect().width / 2) , (HEIGHT / 2) + 72))

    #draw the stars and move their positions...
    for pos in star_positions:
        s = randint(100, 200)
  
        pos[0] = pos[0] - pos[2]
        if pos[0] < 0:
            pos[0] = WIDTH - 1  
        pygame.draw.line(screen, [s, s, s],[pos[0], pos[1]], [pos[0], pos[1]], 1)
    
    screen.blit(surface, surface_rect)
    screen.blit(pad, pad_rect)
    screen.blit(lander, lander_rect)

    if thrust_engaged:
        if not voice.get_busy():
            voice.set_volume(0.1)
            voice.play(sound)

        flame_toggle = not flame_toggle
        if flame_toggle:
            flame1_rect.x = lander_rect.x
            flame1_rect.y = lander_rect.y + 17
            screen.blit(flame1, flame1_rect)
            
        else:
            flame2_rect.x = lander_rect.x
            flame2_rect.y = lander_rect.y + 17
            screen.blit(flame2, flame2_rect)
    else:
        if voice.get_busy():
            sound.stop()
    
    if fuel > 0:
        if fuel > 70:
            colour = GREEN
        elif fuel > 25:
            colour = YELLOW
        else:
            colour = RED
            
        pygame.draw.line(screen, colour, [8, 28], [8 + (fuel * 3), 28], 8)
        label_fuel = myfont.render("FUEL: " + str(fuel) + "%", 1, colour)
        screen.blit(label_fuel, (8, 8))
    else:
        label = myfont.render("FUEL: EMPTY", 1, RED)
        screen.blit(label, (8, 8))           

    label = myfont.render("LEVEL: " + str(multiplier), 1, WHITE)
    screen.blit(label, (WIDTH - (label.get_rect().width + 8), 8))            

    label = myfont.render("SCORE: " + str(int(score)), 1, WHITE)
    screen.blit(label, (WIDTH - (label.get_rect().width + 8), 28))            

    pygame.display.flip()


def round(num):
    if (num > 0):
        return int(num+.5)
    else:
        return int(num-.5)
